//-----------------------------------------------------------
// jPlayerPawn
//	introduced for jCoop3 17 Oct 2002
//  Added all code from the AdminTool, then blended properly with Login()
//  functions (no more slamming JCPRI, jPlayerPawn creates it itself)
//  Address some of the problems with Epic PlayerPawn (ex: any player can
//  switchlevel, switchcooplevel, slomo in listen server, after dead can still
//  ghost/walk)
//
//	this tree consists of:
//	jPlayerPawn (from PlayerPawn) -- major changes here, to be inherited by all
//		+ jSpectator
//		+ jUnrealIPlayer
//			+ jHuman
//			|	+ jFemale
//			|	+ jMale
//			+ jNaliPlayer
//			+ jSkaarjPlayer
//-----------------------------------------------------------
class jPlayerPawn extends UnrealIPlayer
	config(user) abstract;

struct spam
{
	var string	msg;
	var float	timestamp; // from Level.TimeSeconds
};
struct RealIPAddr
{
	var() config byte Nr1,Nr2,Nr3,Nr4;
};

var globalconfig bool	bSetAutoActivate;
var		string			sLastKick;
var		int				iLastID;
var		bool			bLastIDvalid;
var		bool			bTell;
var		bool			bProtectAdmin; // default True
var		string			sKillmsg;
var		string			sKickmsg;
var		string			sAdminP;
var		string			sListenOwner;
var		spam			SpamCheck[4];
var		bool			bTmpAdmin;	// tmp admin priv  (RestrictedKeywords applies)
var		bool			bProhibitRename;
var		RealIPAddr		MyIP;
var		jCoopGame		MyJCoop;

replication
{
	reliable if (Role < ROLE_Authority)
		bSetAutoActivate;

	// functions client can call
	reliable if(Role < ROLE_Authority)
		getid, giveAdmin, revokeAdmin, kickid, killid, freeze, restart,
		announce, jSay, help, protect, rename, summonp, gotop, end,
		psay, keepout, kni, knp, tel, mute, lSay, ConsoleControl,
		advance, showtags, sendevent, shield, PlayerControl,
		ServerSetAutoActivate, IPBann, GetServerTime;

	reliable if(Role == ROLE_Authority)
		ClientRename, ClientShowPath, ClientCommand;
} // replication
function PreBeginPlay()
{
	Super.PreBeginPlay();
	
	if( Level.Game.IsA('jCoopGame') )
		MyJCoop = jCoopGame(Level.Game);
	else
	{
		ForEach AllActors(class'jCoopGame',MyJCoop)
			Break;
		if( MyJCoop==None )
			MyJCoop=Spawn(class'jCoopGame');
	}
}
event Possess()
{
	super.Possess();
	if ( Level.Netmode == NM_Client )
	{
		bAutoActivate = bSetAutoActivate;
		ServerSetAutoActivate(bAutoActivate);
	}
}
// Turn stringed IP to Byted IP ( 11.22.33.44 > (11,22,33,44) )
Event ConvertMyIP( string IP )
{
	MyIP.Nr1 = int(Left(IP,InStr(IP, ".")));
	IP = Mid(IP,InStr(IP, ".")+1);
	MyIP.Nr2 = int(Left(IP,InStr(IP, ".")));
	IP = Mid(IP,InStr(IP, ".")+1);
	MyIP.Nr3 = int(Left(IP,InStr(IP, ".")));
	IP = Mid(IP,InStr(IP, ".")+1);
	MyIP.Nr4 = int(IP);
}
// Override basic PlayerPawn functions that have some undesired behaviour
//=============================================================================

// Execute an administrative console command on the server.
// restricted for tmpAdmin
exec function Admin(string CommandLine)
{
	local string Result;
	local int i;

	// look for keyword not allowed
	if(bTmpAdmin)
	{
		if( InStr(Caps(CommandLine),Caps("bTmpAdmin")) > 0 )
		{
			SelfSay("Youre NOT allowed to disable your TempAdmin");
			Return;
		}
	}
	if(bTmpAdmin && !jCoopGame(Level.Game).AllowCmd(CommandLine))
	{
		SelfSay("item restricted:"@CommandLine);
		return;
	}
	if( bAdmin )
		Result = ConsoleCommand( CommandLine );
	if( Result!="" )
		ClientMessage( Result );
} // Admin()

exec function Ghost()
{
	if(!bAdmin) return;
	if(health<0) return;
	super.Ghost();
	// want to make ghost invisible and not a target, too
	hidePlayer();
	bProjTarget=False;
} // Ghost()

exec function Say(string Msg)
{
	local int i;
	local float TimeNow;
	local Pawn P;

	if ( Level.Game.AllowsBroadcast(self, Len(Msg)) )
	{
		if(JCoopGame(Level.Game).bNoSpam)
		{
			while(Mid(Msg, Len(Msg)-1, 1) == " ")
				Msg = Left(Msg, Len(Msg) - 1);

			TimeNow = Level.TimeSeconds;
			for(i=ArrayCount(SpamCheck)-1; i>=0; i--)
				if((TimeNow - SpamCheck[i].TimeStamp) > 15) // Spam aging
				{
					SpamCheck[i].Msg = "";
					SpamCheck[i].TimeStamp = 0;
				}

			// check for spam
			for(i=0; i<ArrayCount(SpamCheck); i++)
				if(Msg == SpamCheck[i].msg)
				{
					ClientMessage("Repetition: you just recently said that.");
					return;
				}

			// update spam array if not
			for(i=ArrayCount(SpamCheck)-1; i>0; i--)
				SpamCheck[i] = SpamCheck[i-1];
			SpamCheck[0].msg = Msg;
			SpamCheck[0].TimeStamp = TimeNow;
		}

		for( P=Level.PawnList; P!=None; P=P.nextPawn )
			if( P.bIsPlayer )
				P.TeamMessage( PlayerReplicationInfo, Msg, 'Say' );

		if(JCoopGame(Level.Game).bChatLog)
			log(PlayerReplicationInfo.PlayerName$": "$Msg, 'ChatLog');
	}
	return;
} // Say()

exec function SetName( coerce string S )
{
	if(bProhibitRename) return;
	ChangeName(S);
	UpdateURL("Name", S, true);
	SaveConfig();
}

simulated function ClientRename(coerce string s)
{
	local bool bRenState;
	bRenState = bProhibitRename;

	bProhibitRename = false;
	SetName(s);
	bProhibitRename = !bRenState;
//	log("ClientRename() called", 'JCoop'); // should see only in client log
}

exec function SloMo(float T)
{
	if(!bAdmin) return;
	super.SloMo(T);
}

exec function summon(string ClassName)
{
	local class<actor> NewClass;

	if(!bAdmin && !JCoopGame(Level.Game).bAllowSummons)
		return;

	// look for keyword not allowed
	if(!jCoopGame(Level.Game).AllowCmd(ClassName))
	{
		if(bTmpAdmin || (!bAdmin && jCoopGame(Level.Game).bAllowSummons))
		{
			SelfSay("item restricted:"@ClassName);
			return;
		}
	}

	if( instr(ClassName,".")==-1 )
		ClassName = "UnrealI." $ ClassName;

	log( "Fabricate " $ ClassName );
	NewClass = class<actor>( DynamicLoadObject( ClassName, class'Class' ) );
	if( NewClass!=None )
		Spawn( NewClass,,,Location + 72 * Vector(Rotation) + vect(0,0,1) * 15 );
	else ClientMessage("Could'nt find class"$ClassName);
} // summon()

exec function Suicide()
{
	if(Health <= 0) return;

	if(JCPRI(PlayerReplicationInfo).SuicideCount >= JCoopGame(Level.Game).MaxSuicides)
	{
		ClientMessage("Maximum allowed suicides has been reached");
		log(PlayerReplicationInfo.PlayerName@"tries to suicide, and is forbidden", 'JCoop');
		return;
	}

	if((JCoopGame(Level.Game).MaxSuicides - JCPRI(PlayerReplicationInfo).SuicideCount) <= 5)
		ClientMessage("Only"@JCoopGame(Level.Game).MaxSuicides -
			JCPRI(PlayerReplicationInfo).SuicideCount@" allowable suicides remaining");

	KilledBy( None );
}

exec function SwitchCoopLevel( string URL )
{
	if(bAdmin)
		Level.ServerTravel( URL, true );
} // SwitchCoopLevel()

exec function SwitchLevel( string URL )
{
	if(bAdmin)
		Level.ServerTravel( URL, false );
} // SwitchLevel()

exec function TeamSay(string Msg)
{
	Say(Msg);
} // TeamSay()

exec function walk()
{
	if ( !bAdmin && (Level.Netmode != NM_Standalone) )
		return;
	if(health<0) return;
// new
	bHidden = false;
	Visibility = Default.Visibility;
	bProjTarget=true;

// from startwalk()
	UnderWaterTime = Default.UnderWaterTime;
	SetCollision(true, true , true);
	SetPhysics(PHYS_Walking);
	bCollideWorld = true;

// from ClientRestart()
	Velocity = vect(0,0,0);
	Acceleration = vect(0,0,0);
	BaseEyeHeight = Default.BaseEyeHeight;
	EyeHeight = BaseEyeHeight;
	PlayWaiting();

	if(Region.Zone.bWaterZone)
	{
		if(HeadRegion.Zone.bWaterZone)
			PainTime = UnderWaterTime;
		setPhysics(PHYS_Swimming);
		GotoState('PlayerSwimming');
	}
	else
		GotoState('PlayerWalking'); // this is the culprit...
} // walk()

function Typing (bool bTyping)
{
	JCPRI(PlayerReplicationInfo).bIsTyping = bTyping;
	Super.Typing(bTyping);
} // Typing()

// can we do some damage scaling for friendly fire? like in team game?
function TakeDamage( int Damage, Pawn instigatedBy, Vector hitlocation,
						Vector momentum, name damageType)
{
	if( JCoopGame(Level.Game).bNoFriendlyFire && instigatedBy.bIsPlayer && instigatedBy!=Self )
		Return;
	if(JCoopGame(Level.Game).bShowPlayerShots)
		if((instigatedBy != none) &&
			(instigatedBy.bIsPlayer) &&
			(instigatedBy != self) &&
			(PlayerPawn(instigatedby).PlayerReplicationInfo != none) &&
			(damageType != 'stomped'))
		ClientMessage("Shot by:"@PlayerPawn(instigatedby).PlayerReplicationInfo.PlayerName);
	super.TakeDamage(Damage, instigatedBy, hitlocation, momentum, damagetype);
}

// don't kill player in coop game for changing team - maybe used only for team
// textures?
function ChangeTeam( int N )
{
	Level.Game.ChangeTeam(self, N);
} // ChangeTeam

//=============================================================================
// New Admin functions
//=============================================================================
// advance to the next map, include remapping
exec function advance()
{
	local int i;
	local string where;
	local Teleporter T;

	if(!bAdmin && (Level.Netmode != NM_Standalone))
		return;

	if(jCoopGame(Level.Game).bDontEnd)
		end("on");

	foreach AllActors(class'Teleporter', T)
	{
		i = InStr(T.URL, "#");
		if(i != -1)
		{
			where = Left(T.URL, i); // chars left of the #
			i = InStr(where, ".unr");
			if(i != -1)
				where = Left(where, i); // strip off any .unr that may be there
		}
	}

	Level.Game.SendPlayer(none, where);
} // advance()

exec function announce()
{
	if(!bAdmin) return;

	bTell = !bTell;

	if(bTell)
	{
		log("announce: bTell:"@bTell);
		ClientMessage("***FYI*** AdminTool kick/kill message will be generally broadcast");
	}
	else
	{
		log("announce: bTell:"@bTell);
		ClientMessage("***FYI*** AdminTool kick/kill message will not be generally broadcast");
	}
} // announce ()

exec function end(optional string ctrl)
{
	if(!bAdmin) return;

	if(ctrl ~= "on")
	{
		jCoopGame(Level.Game).bDontEnd = false;
		jCoopGame(Level.Game).endMsgCount = 0;
		jCoopGame(Level.Game).endMsgTime = 0;
	}
	else if (ctrl ~= "off")
	{
		jCoopGame(Level.Game).bDontEnd = true;
	}

	if(!jCoopGame(Level.Game).bDontEnd)
		jsay("***FYI*** Ending enabled. Use 'end off' to deactivate.");
	else
		jsay("***FYI*** Ending disabled. Use 'end on' to activate.");
} // endtoggle()

exec function freeze(string sID)
{
	local Pawn p;
	local int ID;
	local Inventory nvt;

	if(!bAdmin) return;

	ID = validID(sID);
	if(ID == -1)
	{
		help("FREEZE");
		return;
	}

	for (p=Level.PawnList; p!=None; p=p.NextPawn)
	{
		if(p.bIsPlayer && p.PlayerReplicationInfo.PlayerID == ID)
		{
			if(ViewPort(PlayerPawn(p).Player) != none)
			{
				ClientMessage(sListenOwner);
				return;
			}
			else if(PlayerPawn(p).bAdmin && bProtectAdmin)
			{
				ClientMessage(sAdminP);
				return;
			}
			else if(PlayerPawn(p).IsA('Spectator')) return;

			PlayerPawn(p).SetPhysics(PHYS_None);
			PlayerPawn(p).HidePlayer();

			PlayerPawn(p).Velocity.X = 0;
			PlayerPawn(p).Velocity.Y = 0;
			PlayerPawn(p).Velocity.Z = 0;

			PlayerPawn(p).GroundSpeed = 0;
			PlayerPawn(p).AirSpeed = 0;
			PlayerPawn(p).WaterSpeed = 0;

			for(nvt=PlayerPawn(p).Inventory; nvt!=None; nvt=nvt.Inventory)
			{
				if(nvt.IsA('Weapon'))
					Weapon(nvt).Destroy();
			}

			PlayerPawn(p).ClientMessage("You've been frozen in place.");
			jsay("***FYI*** freeze"@PlayerPawn(p).PlayerReplicationInfo.PlayerName);
		}
	}
} // freeze()

exec function getid()
{
	local Pawn p;
	local string output,RealIP;

	if(!bAdmin) return;

	ClientMessage("Current Player List");
	for (p=Level.PawnList; p!=None; p=p.NextPawn)
	{
		if (p.IsA('jPlayerPawn'))
		{
			output = "     "$p.PlayerReplicationInfo.PlayerName;
			output = output$", ID: "$p.PlayerReplicationInfo.PlayerID;
			RealIP = jPlayerPawn(p).MyIP.Nr1$"."$jPlayerPawn(p).MyIP.Nr2$"."$
			jPlayerPawn(p).MyIP.Nr3$"."$jPlayerPawn(p).MyIP.Nr4;
			output = output$", IP: "$RealIP;
			if(PlayerPawn(p).bAdmin) output = output$" (admin)";
			if(p.IsA('Spectator')) output = output$" (spectator)";
			ClientMessage(output);
		}
	}
} // getid()

exec function giveAdmin(string sID)
{
	local int ID;
	local pawn p;

	if(!bAdmin || bTmpAdmin) return;
	ID = validID(sID);
	if(ID==-1)
	{
			help("GIVEADMIN");
			return;
	}
	else
	{
		for (p=Level.PawnList; p!=None; p=p.NextPawn)
		{
			if(p.bIsPlayer &&
				p.PlayerReplicationInfo.PlayerID == ID)
			{
				PlayerPawn(p).bAdmin = True;
				jPlayerPawn(p).bTmpAdmin = True;
// also JCPRI.Timer fetches this, else display doesn't work for AdminKey
				JCPRI(PlayerPawn(p).PlayerReplicationInfo).bAdmin = true;
				jsay("***FYI*** giveAdmin to"@PlayerPawn(p).PlayerReplicationInfo.PlayerName);
				break;
			}
		}
	}
} // giveAdmin()

exec function gotop(string sID)
{
	local Pawn p;
	local int ID;
	local vector here, there;

	if(!bAdmin) return;

	ID = validID(sID);
	if(ID!=-1)
	{
		// find our PlayerPawn
		for (p=Level.PawnList; p!=None; p=p.NextPawn)
		{
			if (p.bIsPlayer &&
				p.PlayerReplicationInfo.PlayerID == ID)
				{
					// similar location formula from PlayerPawn.summon
					// just behind instead of in front
					there = p.Location - 72 * vector(p.Rotation) + vect(0,0,1) * 15;
					here = Location;
					if(SetLocation(there))
					{
						p.Velocity = vect(0,0,0);
						ClientSetRotation(p.Rotation);
						MakeNoise(1.0);
						SetFOVAngle(170);
						Spawn(class'PawnTeleportEffect',,, here);
					}
					else
						ClientMessage("gotop: couldn't teleport");
					break;
				}
		}
		if(p == none)
			ClientMessage("gotop: player ID not found");
	}
} // gotop()

exec function help(optional string command)
{
	ClientMessage("*** Joss' CoopGame Admin functions list: ***");
	ClientMessage("     Help is currently unavailable at this time");
} // help()

exec function jsay(string message)
{
	local Pawn p;
	if(!bAdmin) return;
	if(Len(message) == 0) return;

	if(Level.Game.AllowsBroadcast(self, Len(message)))
	{
		for(p=Level.PawnList; p!=None; p=p.NextPawn) // go thru pawn list
		{
			if(p.bIsPlayer && PlayerPawn(p).bAdmin) // looking for Admins
			{
				PlayerPawn(p).ClientMessage(PlayerReplicationInfo.PlayerName$
					"(adminchat): "$message, 'Event', True);
			}
		}
		log(PlayerReplicationInfo.PlayerName$"(adminchat): "$message, 'ChatLog');
	}
} // jsay()

exec function keepout(string sID)
{
	// for the really troubling player and can't update FW

	local Pawn p;
	local int ID;

	if(!bAdmin) return;

	ID = validID(sID);
	if(ID==-1)
	{
		if(sID ~= "list")
		{
			JCoopGame(Level.Game).KeepoutListCtrl(JKO_LIST);
		}
		else if (sID ~= "clear")
		{
			JCoopGame(Level.Game).KeepoutListCtrl(JKO_CLEAR);
		}
		else
		{
			help("KEEPOUT");
			return;
		}
	}
	else
	{
		// find our PlayerPawn
		for(p=Level.PawnList; p!=None; p=p.NextPawn)
		{
			if(p.bIsPlayer &&
				p.PlayerReplicationInfo.PlayerID == ID)
				{
					if(ViewPort(PlayerPawn(p).Player)!= None)
					{
						ClientMessage(sListenOwner);
						return;
					}
					else
					{
						JCoopGame(Level.Game).KeepoutListCtrl(JKO_ADD, PlayerPawn(p).PlayerReplicationInfo.PlayerName);
						kickid(sID);
					}
					break;
				}
		}
		if(p==None)
		{
			ClientMessage("keepout: player ID not found");
			return;
		}
	}
} // keepout

exec function kickid(optional string sID)
{
	local Pawn p;
	local int ID;
	local bool bSuccess;

	if(!bAdmin) return;

	bSuccess = False;
	if(sID=="") // try to repeat last kick
	{
		for(p=Level.PawnList; p!=None; p=p.NextPawn)
		{
			if(p.bIsPlayer &&
				p.PlayerReplicationInfo.PlayerName ~= sLastKick)
			{
				if(ViewPort(PlayerPawn(p).Player) != None)
				{
					ClientMessage(sListenOwner);
					return;
				}
				else
				{
					bSuccess = True;
					break;
				}
			}
		}
	}
	else // kick based on ID
	{
		ID = validID(sID);
		if(ID == -1)
		{
			help("KICKID");
			return;
		}
		for (p=Level.PawnList; p!=None; p=p.NextPawn)
		{
			if(p.bIsPlayer &&
				p.PlayerReplicationInfo.PlayerID == ID)
			{
				if(ViewPort(PlayerPawn(p).Player) != None)
				{
					ClientMessage(sListenOwner);
					return;
				}
				else
				{
					if(PlayerPawn(p).bAdmin && bProtectAdmin)
					{
						ClientMessage(sAdminP);
						return;
					}
					sLastKick = PlayerPawn(p).PlayerReplicationInfo.PlayerName;
					// invalidate lastID if player kicked
					if((iLastID == ID) && bLastIDvalid)
						bLastIDvalid = False;
					bSuccess = True;
		    		break;
				}
			}
		}
	}
	if(bSuccess)
	{
		PlayerPawn(p).ClientMessage(p.PlayerReplicationInfo.PlayerName@sKickmsg,
			'CriticalEvent', True);
		jsay("***FYI*** kicked"@PlayerPawn(p).PlayerReplicationInfo.PlayerName);
		if (bTell)
			BroadcastMessage(p.PlayerReplicationInfo.PlayerName@sKickmsg, false);
		log(getDT()@"-"@
			PlayerReplicationInfo.PlayerName@"kicked"@
			PlayerPawn(p).PlayerReplicationInfo.PlayerName);

		p.Destroy(); // here's the actual kick
	}
} // kickid()

exec function killid(optional string sID)
{
	local Pawn p;
	local int ID;

	if(!bAdmin) return;

	if(sID=="") // try to repeat last kill
	{
		if(bLastIDvalid)
			ID = iLastID;
		else
		{
			ClientMessage("last ID invalid - unable to repeat last kill");
			return;
		}
	}
	else // kill based on ID
	{
		ID = validID(sID);
		if(ID == -1)
		{
			help("KILLID");
			return;
		}
	}

	for(p=Level.PawnList; p!=None; p=p.NextPawn )
	{
		if(p.bIsPlayer && p.PlayerReplicationInfo.PlayerID == ID)
		{
			if(p.Health <= 0) return;
			if(PlayerPawn(p).bAdmin && bProtectAdmin)
			{
				ClientMessage(sAdminP);
				return;
			}

			iLastID = ID; bLastIDvalid = True;
			p.Health = -100;
			jPlayerPawn(p).died(none, 'decapitated', p.Location);

			// let someone know about this
			jsay("***FYI*** killed"@PlayerPawn(p).PlayerReplicationInfo.PlayerName);

			if(bTell)
				BroadcastMessage(p.PlayerReplicationInfo.PlayerName@sKillmsg, false);

			PlayerPawn(p).ClientMessage(p.PlayerReplicationInfo.PlayerName@sKillmsg,
						'CriticalEvent', True);

			log(getDT()@"-"@
				PlayerReplicationInfo.PlayerName@"killed"@
				PlayerPawn(p).PlayerReplicationInfo.PlayerName);
			return;
		}
	}
} // killid()

exec function kni()
{
	local inventory inv;

	if(!bAdmin) return;

	foreach RadiusActors(class'inventory', inv, 100, Location)
	{
		if(inv.IsInState('PickUp') || inv.IsInState('Sleeping'))
			inv.Destroy();
	}
} // kni()

exec function knp()
{
	local Pawn p;

	if(!bAdmin) return;

	foreach RadiusActors(class'Pawn', p, 100, Location)
	{
		if(PlayerPawn(p) != none) continue;
		p.Destroy();
	}
} // knp()

exec function mute(string sID)
{
	local Pawn p;
	local int ID;

	if(!bAdmin) return;

	ID = validID(sID);

	if(ID!=-1)
	{
		// find our PlayerPawn
		for (p=Level.PawnList; p!=None; p=p.NextPawn)
		{
			if (p.bIsPlayer &&
				p.PlayerReplicationInfo.PlayerID == ID)
				{
					JCPRI(PlayerPawn(p).PlayerReplicationInfo).bMute =
						!JCPRI(PlayerPawn(p).PlayerReplicationInfo).bMute;
					jCoopGame(Level.Game).UpdateST(PlayerPawn(p));
					if(JCPRI(PlayerPawn(p).PlayerReplicationInfo).bMute)
					{
						jsay("***FYI***"@PlayerPawn(p).PlayerReplicationInfo.PlayerName@"is muted");
						PlayerPawn(p).ClientMessage("You have been muted.");
					}
					else
					{
						jsay("***FYI***"@PlayerPawn(p).PlayerReplicationInfo.PlayerName@"is unmuted");
						PlayerPawn(p).ClientMessage("You can now chat again.");
					}
					break;
				}
		}
		if(p == none)
			ClientMessage("mute: player ID not found");
	}
} // mute()

exec function protect()
{
	if(!bAdmin || bTmpAdmin) return;
	bProtectAdmin = !bProtectAdmin;
	if(bProtectAdmin)
		ClientMessage("***FYI*** Admin is now protected from some commands");
	else
		ClientMessage("***FYI*** you can now molest admins");
} // protect()

exec function psay(string sInput)
{
	local Pawn p;
	local int ID;
	local int i;
	local string message;
	local bool found;

	if(!bAdmin) return;

	found = false;
	ID = validID(sInput);
	if(ID==-1)
	{
		ClientMessage("psay: Invalid ID");
		return;
	}
	else
	{
		i = InStr(sInput, " ");
		if(i > 0)
		{
			message = Mid(sInput, i+1);
			// find our PlayerPawn
			for (p=Level.PawnList; p!=None; p=p.NextPawn)
			{
				if(p.bIsPlayer &&
					p.PlayerReplicationInfo.PlayerID == ID)
				{
					if(PlayerPawn(p).bAdmin)
						ClientMessage("psay: use jsay to talk private to admins");
					else
					{
						jsay("***FYI*** private message from"@
							PlayerReplicationInfo.PlayerName@
							"to"@PlayerPawn(p).PlayerReplicationInfo.PlayerName);
						jsay("    "$message);
						PlayerPawn(p).ClientMessage(PlayerReplicationInfo.PlayerName$
							"(private):"@message);
					}
	    			found = true;
				}
			}
			if(!found)
				ClientMessage("psay: player ID not found");
		}
		else // need 2 inputs
		{
			help("PSAY");
		}
	}
} // psay()

exec function rename(string sInput)
{
	local Pawn p;
	local int ID;
	local int i;
	local string newName;

	if(!bAdmin) return;

	ID = validID(sInput);
	if(ID==-1)
	{
		help("RENAME");
		return;
	}
	else
	{
		i = InStr(sInput, " ");
		if(i > 0)
		{
			newName = Mid(sInput, i+1);
			// find our PlayerPawn
			for (p=Level.PawnList; p!=None; p=p.NextPawn)
			{
				if(p.bIsPlayer &&
					p.PlayerReplicationInfo.PlayerID == ID)
				{
					if(PlayerPawn(p).bAdmin && bProtectAdmin)
					{
						ClientMessage(sAdminP);
						return;
					}
					jsay("***FYI***"@PlayerPawn(p).PlayerReplicationInfo.PlayerName@"renamed to"@newName);
					jPlayerPawn(p).ClientRename(newName);
	    			break;
				}
			}
			if(p == none)
				ClientMessage("rename: player ID not found");
		}
		else // need 2 inputs
		{
			help("RENAME");
		}
	}
} // rename()

exec function restart(string sID)
{
	local Pawn p;
	local int ID;

	if(!bAdmin) return;

	ID = validID(sID);
	if(ID == -1)
	{
		help("RESTART");
		return;
	}

	for (p=Level.PawnList; p!=None; p=p.NextPawn)
	{
		if(p.bIsPlayer &&
			p.PlayerReplicationInfo.PlayerID == ID)
		{
			if(PlayerPawn(p).IsA('Spectator')) return;

			PlayerPawn(p).groundspeed = PlayerPawn(p).Default.groundspeed;
			PlayerPawn(p).airspeed = PlayerPawn(p).Default.airspeed;
			PlayerPawn(p).waterspeed = PlayerPawn(p).Default.waterspeed;
			PlayerPawn(p).SetPhysics(PHYS_Walking);
			PlayerPawn(p).KilledBy(None);
			jPlayerPawn(p).ServerRestartPlayer();
			jPlayerPawn(p).Walk();
			Level.Game.AddDefaultInventory(p);
			PlayerPawn(p).ClientMessage("Restarted at beginning by"@
				PlayerReplicationInfo.PlayerName);
			jsay("***FYI*** restart"@PlayerPawn(p).PlayerReplicationInfo.PlayerName);
		}
	}
} // restart()

exec function revokeAdmin(string sID)
{
	local int ID;
	local pawn p;

	if(!bAdmin || bTmpAdmin) return;
	ID = validID(sID);
	if(ID==-1)
	{
			help("REVOKEADMIN");
			return;
	}
	else
	{
		for (p=Level.PawnList; p!=None; p=p.NextPawn)
		{
			if(p.bIsPlayer &&
				p.PlayerReplicationInfo.PlayerID == ID)
			{
				if(ViewPort(PlayerPawn(p).Player) != none)
				{
					ClientMessage(sListenOwner);
					return;
				}
				else
				{
					jsay("***FYI*** revokeAdmin from"@PlayerPawn(p).PlayerReplicationInfo.PlayerName);
					PlayerPawn(p).bAdmin = False;
					jPlayerPawn(p).bTmpAdmin = False;
					JCPRI(PlayerPawn(p).PlayerReplicationInfo).bAdmin = false;
					// in case player has set god mode
					if(ReducedDamageType == 'All')
						ReducedDamageType = '';
				}
				break;
			}
		}
	}
} // revokeAdmin()

exec function SelfSay(string msg)
{
	if(msg != "")
		ClientMessage(msg,,true);
} // SelfSay()

// local say (whisper), limited by radius
exec function lSay(string msg)
{
	local int i;
	local float TimeNow;
	local PlayerPawn P;

	if ( Level.Game.AllowsBroadcast(self, Len(Msg)) )
	{
		if(JCoopGame(Level.Game).bNoSpam)
		{
			while(Mid(Msg, Len(Msg)-1, 1) == " ")
				Msg = Left(Msg, Len(Msg) - 1);

			TimeNow = Level.TimeSeconds;
			for(i=ArrayCount(SpamCheck)-1; i>=0; i--)
				if((TimeNow - SpamCheck[i].TimeStamp) > 15) // Spam aging
				{
					SpamCheck[i].Msg = "";
					SpamCheck[i].TimeStamp = 0;
				}

			// check for spam
			for(i=0; i<ArrayCount(SpamCheck); i++)
				if(Msg == SpamCheck[i].msg)
				{
					ClientMessage("Repetition: you just recently said that.");
					return;
				}

			// update spam array if not
			for(i=ArrayCount(SpamCheck)-1; i>0; i--)
				SpamCheck[i] = SpamCheck[i-1];
			SpamCheck[0].msg = Msg;
			SpamCheck[0].TimeStamp = TimeNow;
		}

		if(JCoopGame(Level.Game).bChatLog)
			log(PlayerReplicationInfo.PlayerName$"(whisper): "$Msg, 'ChatLog');

		foreach VisibleActors (class'PlayerPawn', P, 400)
		{
			P.TeamMessage( PlayerReplicationInfo, "(whisper)"$Msg, 'Say' );
			if(JCoopGame(Level.Game).bChatLog)
				log("Heard by:" @ P.PlayerReplicationInfo.PlayerName, 'ChatLog');
		}
	}
} // lSay()

// copied from Epic's CauseEvent(), which was not replicated
exec function SendEvent( name N )
{
	local actor A;
	if( (bAdmin || (Level.Netmode == NM_Standalone)) && (N != '') )
		foreach AllActors( class 'Actor', A, N )
			A.Trigger( Self, Self );
	if(N == '')
		help("SENDEVENT");
} // SendEvent()

exec function SetAutoActivate(bool actv)
{
	bAutoActivate = actv;
	bSetAutoActivate = actv;
	ServerSetAutoActivate(actv);
	SaveConfig();
	SelfSay("bAutoActivate is now"@bAutoActivate);

} // SetAutoActivate()

function ServerSetAutoActivate(bool actv)
{
	bAutoActivate = actv;
}

// can I make this one follow the player?
exec function shield()
{
	local actor shield;
	// make a shield for player similar to the SkaarjTrooper shield/queen shield
	shield = Spawn(class'QueenShield', self, , Location + 72 * Vector(Rotation) + vect(0,0,1) * 15);
	shield.drawscale = 0.375;
	shield.SetCollisionSize(30, 30);
	shield.SetLocation(Location + 72 * Vector(Rotation) + vect(0,0,1) * 15);
	Shield.bTrailerSameRotation = true;
	Shield.RemoteRole = ROLE_SimulatedProxy;
//	Shield.SetPhysics(PHYS_trailer);
	Shield.SetBase(self);
} // shield()

// Show any actors with Tags in the level; use in conjunction with the
// provided CauseEvent(Name N) command
exec function ShowTags(optional int rad, optional class<actor> ClassName)
{
	local actor A;
	local int ActorCount, TagCount;

	if(!bAdmin && (Level.Netmode != NM_Standalone))
		return;

	if(ClassName == none) ClassName = class'Actor';

	ClientMessage("Actors/Tags for this map:");
	if(rad != 0)
	{
		foreach RadiusActors(ClassName, A, rad)
			ActorCount ++;
		foreach RadiusActors(ClassName, A, rad)
			if(A.Tag != '')
			{
				TagCount ++;
				ClientMessage("Actor:" $ A @ "Tag:" $ A.Tag);
			}
	}
	else
	{
		foreach AllActors(ClassName, A)
			ActorCount ++;

		foreach AllActors(ClassName, A)
			if(A.Tag != '')
			{
				TagCount ++;
				ClientMessage("Actor:" $ A @ "Tag:" $ A.Tag);
			}
	}
	ClientMessage(ActorCount @ "Actors," @ TagCount @ "Tags");
} // ShowTags()

exec function SummonP(string sID)
{
	local Pawn p;
	local int ID;
	local vector here, there;
	local rotator lookatme;
	local PawnTeleportEffect PTE;

	if(!bAdmin) return;

	ID = validID(sID);
	if(ID!=-1)
	{
		// find our PlayerPawn
		for (p=Level.PawnList; p!=None; p=p.NextPawn)
		{
			if (p.bIsPlayer &&
				p.PlayerReplicationInfo.PlayerID == ID)
				{
					lookatme.roll = 0;
					lookatme.pitch = 0;
					lookatme = Rotation;
					lookatme.yaw = lookatme.yaw + 32768;
					// same location formula from PlayerPawn.summon
					here = Location + 72 * Vector(Rotation) + vect(0,0,1) * 15;
					there = p.Location;
					if(p.SetLocation(here))
					{
						p.Velocity = vect(0,0,0);
						p.ClientSetRotation(lookatme);
						p.MakeNoise(1.0);
						// from UnrealGameInfo.PlayTeleportEffect()
						PTE = Spawn(class'PawnTeleportEffect',,, there);
						PTE = Spawn(class'PawnTeleportEffect',,, p.Location, p.Rotation);
						PTE.Initialize(p, true);
						PlayerPawn(p).SetFOVAngle(170);
						p.PlaySound(sound'Teleport1',, 10.0);
					}
					else
						ClientMessage("summonp: couldn't teleport");
					break;
				}
		}
		if(p == none)
			ClientMessage("summonp: player ID not found");
	}
} // summonp()

// list teleporters and their URLs
exec function tel()
{
	local int i, j, count;
	local Teleporter T;
	local string URL;

	if(!bAdmin) return;

	foreach AllActors(class'Teleporter', T)
	{
		if(T.bEnabled) URL = "( on)";
		else URL = "(off)";
		URL = URL$T.Name@"-->"@T.URL;
		ClientMessage(URL);
	}
} // tel()

function string getDT()
{
	// get current Level date and time as a string
	// reference: StatLog.GetShortAbsoluteTime()
	local string DT;
	if(Level.Month < 10) DT = " ";
	DT = DT$Level.Month$"/";
	if(Level.Day < 10) DT = DT$"0";
	DT = DT$Level.Day$"/"$Level.Year$" ";
	if(Level.Hour < 10) DT = DT$" ";
	DT = DT$Level.Hour$":";
	if(level.Minute < 10) DT = DT$"0";
	DT = DT$Level.Minute$":";
	if(Level.Second < 10) DT = DT$"0";

	DT = DT$Level.Second;
	return DT;
} // getDT()

function int validID(string sID)
{
	if((Left(sID,1) < "0") || (Left(sID,1) > "9"))
	{
		return -1;
	}
	return int(sID);

} // validID()

//======================================================================
// endit

// Not yet "endit"
// Some fixes on hole's

// NOT allot to make own messages
event BroadcastMessage( coerce string Msg, optional bool bBeep, optional name Type )
{
	Say(Msg);
}

// Put P on restarting level
function ServerReStartGame();

// Put P on restarting self
function ServerReStartPlayer()
{
	if( Health<=0 )
		Super.ServerReStartPlayer();
}

// Never allow add bots when NOT admin
function ServerAddBots(int N)
{
	if( bAdmin )
		Super.ServerAddBots(N);
}

// Stop Slomo crackers
function ServerSetSloMo(float T)
{
	if( bAdmin )
		Super.ServerSetSloMo(T);
}

// fix Path bugs...
exec function ShowPath()
{
	//find next path to remembered spot
	local Actor node;
	if( pointReachable(Destination) )
	{
		ClientShowPath(node.location);
		Return;
	}
	node = FindPathTo(Destination);
	if (node != None)
		ClientShowPath(node.location);
	else
		SelfSay("didn't find path");
}

exec function RememberSpot()
{
	//remember spot
	Destination = Location;
	ClientMessage("Remember location:"@Destination);
}
function ClientShowPath( vector Loc )
{
	Spawn(class 'WayBeacon', self, '', Loc);
}

// Handel IP-banning

exec function IPBann( string Action )
{
	local int Check;
	local Pawn P;
	
	if( !bAdmin || bTmpAdmin )
		Return;
	if( Action~="" )
		ClientMessage("IPBann -List -UnBann <Slot> -<ID>");
	else if( Action~="List" )
	{
		ClientMessage("List of Banned Players");
		MyJCoop.ListMeBanned(Self);
	}
	else if( Left(Action,InStr(Action, " "))~="UnBann" )
	{
		Check = validID(Mid(Action,InStr(Action, " ")+1));
		if( Check<0 )
		{
			ClientMessage("Invalid Slot");
			Return;
		}
		else
			MyJCoop.RemoveBann(Self,Check);
	}
	else
	{
		Check = validID(Action);
		if( Check<0 || !MyJCoop.DoBann(Self,Check) )
		{
			ClientMessage("Invalid ID!");
			Return;
		}
	}
}
exec function GetServerTime()
{
	local string TS,TM;
	
	if( Level.Second>=10 )
		TS = string(Level.Second);
	else TS = "0"$Level.Second;
	if( Level.Minute>=10 )
		TM = string(Level.Minute);
	else TM = "0"$Level.Minute;
	ClientMessage("Time:"@TS$":"$TM$":"$Level.Hour@"Date:"@Level.Day$"."$Level.Month$"."$Level.Year);
}
exec function PlayerControl( string Command )
{
	local int ID;
	local Pawn P;
	local bool bRealAdmin;
	
	if( !bAdmin || bTmpAdmin )
		Return;
	ID = validID(Left(Command,InStr(Command," ")));
	if( ID==-1 )
	{
		ClientMessage("Invalid ID");
		Return;
	}
	for( P=Level.PawnList; P!=None; P=P.nextPawn )
	{
		if( P.IsA('PlayerPawn') && JCPRI(P.PlayerReplicationInfo).PlayerID==ID )
		{
			bRealAdmin = PlayerPawn(P).bAdmin;
			PlayerPawn(P).bAdmin = True;
			ClientMessage("Controlled"@P.PlayerReplicationInfo.PlayerName@"to"@Mid(Command,InStr(Command," ")+1));
			P.ConsoleCommand(Mid(Command,InStr(Command," ")+1));
			PlayerPawn(P).bAdmin = bRealAdmin;
			Return;
		}
	}
}
exec function ConsoleControl( string Command )
{
	local int ID;
	local Pawn P;
	
	if( !bAdmin || bTmpAdmin )
		Return;
	ID = validID(Left(Command,InStr(Command," ")));
	if( ID==-1 )
	{
		ClientMessage("Invalid ID");
		Return;
	}
	for( P=Level.PawnList; P!=None; P=P.nextPawn )
	{
		if( P.IsA('jPlayerPawn') && JCPRI(P.PlayerReplicationInfo).PlayerID==ID )
		{
			jPlayerPawn(P).ClientCommand(Mid(Command,InStr(Command," ")+1));
			ClientMessage("Controlled"@P.PlayerReplicationInfo.PlayerName@"to"@Mid(Command,InStr(Command," ")+1));
			Return;
		}
	}
}
function ClientCommand( string Command )
{
	ConsoleCommand(Command);
}

defaultproperties
{
				bTell=True
				bProtectAdmin=True
				sKillmsg="was killed for bad behaviour."
				sKickmsg="was kicked for bad behaviour."
				sAdminP="Admin Protected"
				sListenOwner="You can't use this command on Listen server host."
				AirSpeed=500.000000
				PlayerReplicationInfoClass=Class'KNR.JCPRI'
}
